################################################ Grad-seq browser with multiple selection ############################################################


import dash
from dash.dependencies import Input, Output, State
import dash_table
import dash_core_components as dcc
import dash_html_components as html
import dash_bootstrap_components as dbc
import plotly.graph_objs as go
import numpy as np
import pandas as pd
import json

# Keep this out of source code repository - save in a file or a database


app = dash.Dash(__name__, external_stylesheets=[dbc.themes.BOOTSTRAP])

server = app.server

app.config["suppress_callback_exceptions"] = True


#refs
fractions = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21']
zero = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]

colors = {
    'background': '#FFFFFF',
    'text': '#000000'
}

# import data and search for containing stringn

dtype_dic = {'ID':str, 'locus':str, 'gene':str, 'product':str, 'GO-terms':str, 'Pfam-IDs':str, 'mass':float}

n2g = pd.read_csv("data/n2g_SeT_Sp_Ec.csv", header=0, index_col=0, usecols=[0,*range(1,22)], sep=';')

pl_set = pd.read_csv("data/SeT_Protein.csv", header=0, index_col = 0, usecols = [*range(0,10)], sep=';', dtype = dtype_dic)
pl_ec = pd.read_csv("data/Ec_Protein.csv", header=0, index_col = 0, usecols = [*range(0,10)], sep=';', dtype = dtype_dic)
pl_sp = pd.read_csv("data/Sp_Protein.csv", header=0, index_col = 0, usecols = [*range(0,10)], sep=';', dtype = dtype_dic)

pl_empty=[]
pl_empty = pd.read_csv("data/empty_table.csv", header=0, index_col = 0, usecols = [*range(0,10)], sep=';', dtype = dtype_dic)

pl = pl_ec

rna_set = pd.read_csv("data/SeT_RNA.csv", header=0, index_col = 0, usecols = [*range(0,10)], sep=';', dtype = dtype_dic)
rna_ec = pd.read_csv("data/Ec_RNA.csv", header=0, index_col = 0, usecols = [*range(0,10)], sep=';', dtype = dtype_dic)
rna_sp = pd.read_csv("data/Sp_RNA.csv", header=0, index_col = 0, usecols = [*range(0,10)], sep=';', dtype = dtype_dic)

new_clicks = 0
rm_clicks = 0

#adjust the formating of wrongly formatted values
pl_set['mass']=pl_set['mass'].map("{:,.2f}".format)
rna_set['mass']=rna_set['mass'].map("{:,.2f}".format)
pl_sp['mass']=pl_sp['mass'].map("{:,.2f}".format)
rna_sp['mass']=rna_sp['mass'].map("{:,.2f}".format)
pl_ec['mass']=pl_ec['mass'].map("{:,.2f}".format)
rna_ec['mass']=rna_ec['mass'].map("{:,.2f}".format)


# Layout
app.layout =    html.Div([
                html.Div([
                dcc.Markdown('''## A Grad-seq map of RNA and protein complexes in *Escherichia coli*, *Salmonella enterica*, *Streptococcus pneumoniae*''', style={'textAlign': 'center', 'marginTop':'30px'}),
                dcc.Markdown('''**Vogel lab**, *Helmholtz Institute for RNA-based Infection Research/Institute for Molecular Infection Biology, 97080 Würzburg, Germany*''', style={'textAlign': 'center'}),
                
                # database selection
                
                dcc.Dropdown(id="dynamic-dropdown", options=[
                                {'label': 'Proteins S. enterica SL1344 (20 fract.), Smirnov et al. 2016', 'value': 'pset'},
                                {'label': 'Proteins Escherichia coli, (21 fract.), Hör et al. 2020', 'value': 'pec'},
                                {'label': 'Proteins S. pneumoniae (21 fract.), Hör et al. 2020', 'value': 'psp'},
                                {'label': 'RNAs S. enterica SL1344 (20 fract.), Smirnov et al. 2016', 'value': 'rset'},
                                {'label': 'RNAs E. coli (21 fract.), Hör et al. 2020', 'value': 'rec'},
                                {'label': 'RNAs S. pneumoniae (21 fract.), Hör et al. 2020', 'value': 'rsp'}
                                ],
                                placeholder="Select a Grad-seq database",
                                className ="row",
                        ),  
                
                # table with search field

                html.Div(
                dbc.CardBody([
                html.Div([dcc.Markdown('''**ENTER YOUR SEARCH WORD HERE AND SELECT product FROM TABLE:**''', style={'width':'50%', 'marginTop':'12px', 'marginLeft':'8px'}), dbc.Input(id='st', value='', placeholder='gene, product, locus, ID, Pfam-ID', type='text', style={'width':'45%'})], style={'display': 'flex', 'justify-content': 'center', 'align-items': 'center','backgroundColor':'rgb(245,245,245)'}, className="row"),
                html.Div(html.Div([dash_table.DataTable(
                id='data_table1',
                style_data={
                'whiteSpace': 'normal',
                'height': 'auto',
                'if':{'row_index': 'odd'},
                'backgroundColor': 'rgb(248, 248, 248)'
                },
                style_table={
                'maxHeight': '400px',
                'maxWidth': '100%',
                'overflowY': 'scroll',
                'overflowX': 'hidden'
                },
                columns=[
                {"name": i, "id": i, "deletable": False, "selectable": False} for i in pl.columns
                ],
                style_cell_conditional=[
                {'if': {'column_id': 'organism'},
                'width': '10%','textAlign': 'right'},
                {'if': {'column_id': 'type'},
                'width': '5%','textAlign': 'right'},
                {'if': {'column_id': 'ID'},
                'width': '10%','textAlign': 'right'},
                {'if': {'column_id': 'locus'},
                'width': '10%','textAlign': 'center'},
                {'if': {'column_id': 'gene'},
                'width': '10%','textAlign': 'center', 'font-style':'italic', 'font-weight':'bold'},
                {'if': {'column_id': 'product'},
                'width': '18%','textAlign': 'left'},
                {'if': {'column_id': 'GO-terms'},
                'width': '25%','textAlign': 'left'},
                {'if': {'column_id': 'Pfam-IDs'},
                'width': '7%','textAlign': 'center'},
                {'if': {'column_id': 'mass'},
                'width': '10%','textAlign': 'left'},
                ],
                style_cell={
                'overflow': 'hidden',
                'textOverflow': 'native',
                'maxWidth': 0,
                'whiteSpace': 'normal',
                },
                editable=False,
                filter_action="native",
                sort_action="native",
                row_deletable=False,
                selected_columns=[],
                selected_rows=[],
                page_action="native",
                page_current= 0,
                style_as_list_view=True,
                )], style={'width':'100%'}), className="row"),
                ], style={'width':'100%'}), className="row", style={'backgroundColor':'rgb(245,245,245)'}),

                # Display selected item
                html.H5('', id='selected_POI', style={'display': 'none'}),
                dcc.Graph(id='selection', figure={'data':[go.Bar(x=fractions, y=zero, text=fractions, marker_color = 'black')], 'layout':go.Layout(xaxis = {'title':''}, yaxis = {'range': [0, 1]}, height=250, margin={'l': 0, 'r': 0, 't': 5, 'b': 4}, bargap=0)}, config={ 'modeBarButtonsToRemove': ['sendDataToCloud', 'pan2d', 'zoomIn2d', 'zoomOut2d', 'autoScale2d', 'resetScale2d', 'hoverCompareCartesian', 'hoverClosestCartesian', 'toggleSpikelines', 'zoom2d', 'select2d', 'lasso2d',], 'DisplayModeBar': True, 'displaylogo': False, 'toImageButtonOptions': {'format': 'svg', 'filename': 'Grad-seq_browser',}}, className="row"),
                # Button to add to second table with timestamp set to 0

                html.Div([dbc.Button("add selected item to the list for display", id="add_to_list", color="primary", n_clicks_timestamp='0', className="mr-1", style={'width':'100%'})], className="row"),

                # second table

                html.Div(
                dbc.CardBody([
                html.Div(html.Div([dash_table.DataTable(
                id='data_table2',
                style_data={
                #'whiteSpace': 'normal',
                'height': 'auto',
                'if':{'row_index': 'odd'},
                'backgroundColor': 'rgb(248, 248, 248)'
                },
                style_table={
                'maxHeight': '300px',
                'overflowY': 'scroll',
                'overflowX': 'hidden'
                },
                columns=[
                {"name": i, "id": i, "deletable": False, "selectable": False} for i in pl_empty.columns
                ],
                style_cell_conditional=[
                {'if': {'column_id': 'organism'},
                'width': '10%','textAlign': 'right'},
                {'if': {'column_id': 'type'},
                'width': '5%','textAlign': 'right'},
                {'if': {'column_id': 'ID'},
                'width': '10%','textAlign': 'right'},
                {'if': {'column_id': 'locus'},
                'width': '10%','textAlign': 'center'},
                {'if': {'column_id': 'gene'},
                'width': '10%','textAlign': 'center', 'font-style':'italic', 'font-weight':'bold'},
                {'if': {'column_id': 'product'},
                'width': '18%','textAlign': 'left'},
                {'if': {'column_id': 'GO-terms'},
                'width': '25%','textAlign': 'left'},
                {'if': {'column_id': 'Pfam-IDs'},
                'width': '7%','textAlign': 'center'},
                {'if': {'column_id': 'mass'},
                'width': '10%','textAlign': 'left'},
                ],
                style_cell={
                'overflow': 'hidden',
                'textOverflow': 'ellipsis',
                'maxWidth': 0,
                #'whiteSpace': 'normal',
                },
                editable=False,
                filter_action="none",
                sort_action="custom",
                row_deletable=False,
                selected_columns=[],
                selected_rows=[],
                page_action="native",
                page_current= 0,
                style_as_list_view=True,
                )], style={'width':'100%'}), className="row"),
                ]), className="row", style={'backgroundColor':'rgb(245,245,245)'}),
                # select style pf representation and button to remove from second table
                html.P(),
                html.Div([
                    html.Div(dcc.Dropdown(id="style-dropdown", options=[
                                {'label': 'separate bars', 'value': 'bars'},
                                {'label': 'merged smooth lines', 'value': 'merged'},
                                {'label': 'heatmaps', 'value': 'heatmaps'},
                                ],
                                placeholder="Select a representation",
                                style={'width':'100%'}),
                            className ="col-sm-8",
                        ),
                    # Button to remove item from list with timestamp
                    html.Div(dbc.Button("remove selected item", id="remove_from_list", color="danger", n_clicks_timestamp='0', style={'width':'100%'}), 
                    className="col-sm-4")

                ], style={'width':'100%'}, className="row no-gutters"),
                html.P(),
          

                # represent the data
                html.Div([html.Div(id="plots_all")], style = {'display': 'inline-block', 'width': '100%'}),
                html.Div([html.Div(id="hidden_Div")], style = {'display': 'inline-block', 'width': '100%', 'visibility': 'hidden'}),


                # Affiliations Logos and Links
                html.Div([html.A([html.Img(src=app.get_asset_url('HIRI_Logo.svg'),style={'height':'60px','max-width':'50%','marginTop':'15px','marginLeft':'30px', 'marginRight':'30px', 'marginBottom':'80px'})], href="https://www.helmholtz-hzi.de/de/das-hzi/standorte/hiri-helmholtz-institut-fuer-rna-basierte-infektionsforschung/das-hiri/"), 
                          html.A([html.Img(src=app.get_asset_url('UNI_WB_Logo.svg'),style={'height':'60px','max-width':'30%','marginTop':'15px','marginLeft':'30px', 'marginRight':'30px', 'marginBottom':'80px'})], href="https://www.uni-wuerzburg.de/startseite/"),
                          html.A([html.Img(src=app.get_asset_url('zoom_instr.svg'),style={'height':'60px','max-width':'20%','marginTop':'15px','marginLeft':'30px', 'marginRight':'30px', 'marginBottom':'80px', 'align-items':'right'})], href="")]),
                html.Div([html.H6("coded in Python, with Pandas, Plotly, and Dash by M. Gerovac")], style={'marginBottom':'80px', 'font-style':'italic','align-items':'right'}, className="row"),
                ], style={'width': '90%', 'max-width':'1100px'})], className="row", style={'width': '100%', 'justify-content': 'center', 'align-items': 'center', 'backgroundColor':'rgb(255,255,255)'  })


# Search of POI and table generation
@app.callback(Output(component_id='data_table1', component_property='data'),
             [Input('st', 'value'),
              Input('dynamic-dropdown', 'value')])
def update_table(st_value, dd_value):
    print(dd_value)
    database_frame=[]
    if dd_value is None:
        database_frame = pl_ec
    elif dd_value == 'pset':
        database_frame = pl_set
    elif dd_value == 'pec':
        database_frame = pl_ec
    elif dd_value == 'psp':
        database_frame = pl_sp
    elif dd_value == 'rset':
        database_frame = rna_set
    elif dd_value == 'rec':
        database_frame = rna_ec
    elif dd_value == 'rsp':
        database_frame = rna_sp

    pls = database_frame[database_frame['gene'].str.contains(st_value, case=False) | database_frame['product'].str.contains(st_value, case=False) | database_frame['Pfam-IDs'].str.contains(st_value, case=False) | database_frame['locus'].str.contains(st_value, case=False) | database_frame['ID'].str.contains(st_value, case=False)]
    return pls.to_dict('records')


# Select a POI from data-table1
@app.callback(Output('selected_POI', 'children'),
             [Input('data_table1', 'active_cell'),
              Input ('data_table1', 'derived_viewport_data')])
def update_bars(position, data):
        if position is not None and data != []:
            #print(data)
            #print(position)
            row = position['row']
            search_ID = data[row]['ID']
            return search_ID
        else:
                search_ID="none"
                return search_ID

# Create the bar diagram of the selected item
@app.callback(Output('selection', 'figure'),
             [Input('selected_POI', 'children')])
def update_bars_ref(search_ID):
    bar = n2g.loc[search_ID]
    return {'data':[go.Bar(x=fractions, y=bar, marker_color = 'black')], 'layout':go.Layout(xaxis = {'title':''}, yaxis = {'range': [0, 1]}, height=250, margin={'l': 0, 'r': 0, 't': 5, 'b': 4}, bargap=0)}




### THE BUTTON CLICK HAS TO ADD THE SELECTED ITEM FROM ONE TABLE TO THE OTHER

@app.callback(
    Output(component_id='data_table2', component_property='data'), 
    [Input("add_to_list", "n_clicks_timestamp"),
     Input("remove_from_list", "n_clicks_timestamp")],
    [State('data_table1', 'active_cell'),
     State('data_table1', 'derived_viewport_data'),
     State('data_table2', 'active_cell'),
     State('data_table2', 'derived_viewport_data')])
def action_on_click(add_clicks, delete_clicks, position, data, position2, data2):
    table = pd.DataFrame.from_dict(data2)
    if int(add_clicks) > int(delete_clicks) and position is not None: # The select button was pressed
        row = position['row']
        table = table.append(data[row], ignore_index=True)
        return table.to_dict('records')
    elif delete_clicks > add_clicks and position2 is not None:  # The deselect button
        row = position2['row']
        table = table.drop(table.index[row])
        return table.to_dict('records')
    else: # Should only reach here on page load, so add_clicks = remove_clicks, its OK to reset since it should be empty
        return table.to_dict('records')

 

# Create the graphs

##### HERE WE WOULD NEED MULTIPLE TABS OR SELECTION CRITERIA FOR BAR OVERLAYED AND HEATMAPS

@app.callback(Output('plots_all', 'children'),
             [Input('style-dropdown', 'value'),
              Input('data_table2', 'derived_viewport_data')])
def update_plots(stylemod, data2):
    print(stylemod)
    table = pd.DataFrame.from_dict(data2)
    global n2g
    if stylemod == 'bars' or stylemod == None:
        lis=[]        
        lis.append(html.Div(html.Img(src=app.get_asset_url('fract2.svg'), style={'width':'97%'}), className="row justify-content-center"))

        for i in range(len(table)):
            bar_id2 = table.iloc[i]['ID']
            product_organism = table.iloc[i]['organism']
            product_type = table.iloc[i]['type']
            product_name = table.iloc[i]['product']
            product_gene = table.iloc[i]['gene']
            if product_organism is None:
                product_organism = str()
            if product_type is None:
                product_type = str()
            if product_name is None:
                product_name = str()
            product_gene = table.iloc[i]['gene']
            if product_gene is None:
                product_gene = str()
            bars = n2g.loc[bar_id2]
            product_title = str(product_organism) + " - " + str(product_type) + " - " + str(product_gene) + " - " + str(product_name)
            #print(bars)
            lis.append(dcc.Graph(figure={'data':[go.Bar(x=fractions, y=bars, marker_color = 'black')], 'layout':go.Layout(title = product_title, xaxis = {'visible':True, 'showticklabels':False, 'showline':True,'title':''}, yaxis = {'visible':False, 'showline':False, 'range': [0, 1]}, height=200, margin={'l': 6, 'r': 6, 'b':12}, bargap=0)}, config={ 'modeBarButtonsToRemove': ['sendDataToCloud', 'pan2d', 'zoomIn2d', 'zoomOut2d', 'autoScale2d', 'resetScale2d', 'hoverCompareCartesian', 'hoverClosestCartesian', 'toggleSpikelines', 'zoom2d', 'select2d', 'lasso2d',], 'DisplayModeBar': True, 'displaylogo': False, 'toImageButtonOptions': {'format': 'svg', 'filename': 'Grad-seq_browser',}}))
        return lis

    elif stylemod == 'merged':
        traces = []
        height = 400
        #populate the traces
        for i in range(len(table)):
            ident = table.iloc[i]['ID']
            product_name = table.iloc[i]['product']
            trace = n2g.loc[ident]
            #print(id)
            #print(product_name)
            #print(trace)
            traces.append(go.Scatter(
                x=fractions,
                y=trace,
                mode='lines+markers',
                marker = {'size':15},
                name=product_name
            ))
            height = height +27
        #make a graph
        return (html.Div(html.Img(src=app.get_asset_url('fract2.svg'), style={'width':'77%'}), className="row justify-content-center"), dcc.Graph(figure={'data':traces, 'layout':go.Layout(legend = {'orientation':'h','y':-0.15}, title = "merged plot", xaxis = {'visible':True, 'showticklabels':False, 'showline':True,'title':''}, yaxis = {'visible':False, 'showline':False, 'range': [0, 1.1]}, height=height, width=1100)}, config={ 'modeBarButtonsToRemove': ['sendDataToCloud', 'pan2d', 'zoomIn2d', 'zoomOut2d', 'autoScale2d', 'resetScale2d', 'hoverCompareCartesian', 'hoverClosestCartesian', 'toggleSpikelines', 'zoom2d', 'select2d', 'lasso2d',], 'DisplayModeBar': True, 'displaylogo': False, 'toImageButtonOptions': {'format': 'svg', 'filename': 'Grad-seq_browser',}}))
    
    elif stylemod == 'heatmaps':
        table_0 = []
        table_1 = []
        table_2 = []

        table_0.append(html.Div("", style={'width':260,'height':40, 'font-size': 'medium', 'text-align': 'center','display': 'flex', 'backgroundColor':'rgb(255,255,255)'}))
        table_0.append((html.Div("1", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("2", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("3", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("4", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("5", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("6", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("7", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("8", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("9", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("10", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("11", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("12", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("13", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("14", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("15", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("16", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("17", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))            
        table_0.append((html.Div("18", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("19", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("20", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))
        table_0.append((html.Div("P", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-bottom': '2px solid black', 'border-top': '2px solid black', 'text-align': 'center'})))

        #populate the traces
        for i in range(len(table)):
            id2 = table.iloc[i]['ID']
            gene = table.iloc[i]['gene']
            prtype = table.iloc[i]['type']
            organi = table.iloc[i]['organism']
            text = str(organi)+ " " + str(id2)+" "+str(gene)
            trace = n2g.loc[id2]

            table_1.append(html.Div(text, style={'width':260,'height':40, 'font-size': 'medium', 'text-align': 'left', 'align-items': 'center','display': 'flex', 'backgroundColor':'rgb(255,255,255)'}))
            for a in range(len(trace)):
                value = float(trace[a])
                #print(value)
                if prtype == "RNA":
                    color_cell = 'rgb('+str(255)+','+str(255-value*255)+','+str(255-value*255)+')'
                else: 
                    color_cell = 'rgb('+str(255-value*255)+','+str(255-value*255)+','+str(255)+')'
                #print(color_cell)
                if value == -0.0001:
                    table_1.append((html.Div("", style={'width':40, 'height':40, 'backgroundColor':'white', 'border-style': 'none', 'border-color': 'black', 'border-width': '1px'}))) 
                else:
                    table_1.append((html.Div("", style={'width':40, 'height':40, 'backgroundColor':color_cell, 'border-style': 'solid', 'border-color': 'black', 'border-width': '1px'})))
            table_2.append(html.Div(table_1, className="row"))
            table_1 = []
        return [html.Div(table_0, style={'display': 'flex', 'justify-content': 'center', 'align-items': 'center'}, className="row"), 
                html.Div(table_2, style={'display': 'flex', 'justify-content': 'center', 'align-items': 'center'}, className="row"),
          ]

if __name__ == '__main__':
    app.run_server(host='0.0.0.0', port=8050)










